//+------------------------------------------------------------------+
//|                                                    iUniTrend.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.1"
#property indicator_separate_window
#property indicator_minimum -2
#property indicator_maximum 3
#property indicator_buffers 4
#property indicator_plots   2
//--- plot up
#property indicator_label1  "up"
#property indicator_type1   DRAW_ARROW
#property indicator_color1  clrBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot dn
#property indicator_label2  "dn"
#property indicator_type2   DRAW_ARROW
#property indicator_color2  clrRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

#include <UniTrend/UniTrendDefines.mqh>
#include <UniTrend/UniTrendIndicators.mqh>

bool                 UseGUI      =  false;

input   EAlerts               Alerts      =  Alerts_off;
input   EType                 Type        =  Type_ADXW_PDIMDI; 

bool                 UseDefault  =  false;
bool                 KeepPrev    =  false;

input   int                   f_period1   =  12;
input   int 	               f_shift1    =  0;
input   int 	               f_period2   =  26;
input   int 	               f_shift2    =  0;
input   int 	               f_period3   =  9;
input   int 	               f_shift3    =  0;
input   ENUM_MA_METHOD 	      f_method    =  MODE_SMA;
input   ENUM_APPLIED_PRICE 	f_price     =  PRICE_CLOSE;
input   ENUM_APPLIED_VOLUME   f_volume    =  VOLUME_TICK;
input   ENUM_STO_PRICE 	      f_sto_price =  STO_LOWHIGH;
input   double 	            f_level     =  0;
input   int 	               f_dot2shift =  1;
input   double 	            f_step      =  0.02;
input   double 	            f_maximum   =  0.2;
input   int 	               s_period1   =  24;
input   int 	               s_shift1    =  0;
input   int 	               s_period2   =  52;
input   int 	               s_shift2    =  0;
input   int 	               s_period3   =  18;
input   int                   s_shift3    =  0;
input   ENUM_MA_METHOD 	      s_method    =  MODE_SMA;
input   ENUM_APPLIED_PRICE 	s_price     =  PRICE_CLOSE;  
input   bool                  Auto5Digits =  true; 

EType _Type;
SExtParams Parameters;
CUniTrend * Indicator;

string ShortName;

double         up[];
double         dn[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
  
   if(Auto5Digits && (Digits()==3 || Digits()==5)){
      Parameters.mult=10;
   }
   else{
      Parameters.mult=1;
   }  
  
   PrepareParameters(); 
  
   LoadIndicator();
   
   if(!Indicator.CheckHandles()){
      Alert("Error loading "+Indicator.Name());
      return(INIT_FAILED);
   }      

   ShortName=Indicator.Name();

   if(!UseGUI){
      PrintColl("Parameters matching:",Indicator.Help());
   }   
  
//--- indicator buffers mapping
   SetIndexBuffer(0,up,INDICATOR_DATA);
   SetIndexBuffer(1,dn,INDICATOR_DATA);
//--- setting a code from the Wingdings charset as the property of PLOT_ARROW
   PlotIndexSetInteger(0,PLOT_ARROW,233);
   PlotIndexSetInteger(1,PLOT_ARROW,234);
   
   IndicatorSetInteger(INDICATOR_DIGITS,0);
   IndicatorSetString(INDICATOR_SHORTNAME,ShortName);
//---
   return(INIT_SUCCEEDED);
}
  
void OnDeinit(const int reason){
   if(CheckPointer(Indicator)==POINTER_DYNAMIC){
      delete Indicator;
   }
}    
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {

   int rv=Indicator.Calculate(rates_total,prev_calculated,up,dn);
   
   if(Alerts!=Alerts_off){
      static datetime tm0=0;
      static datetime tm1=0;
      if(tm0==0){
         tm0=time[rates_total-1];
         tm1=time[rates_total-1];
      }
      string mes="";
      
      if(up[rates_total-Alerts]!=EMPTY_VALUE && 
         up[rates_total-Alerts-1]==EMPTY_VALUE && 
         tm0!=time[rates_total-1]
      ){
         tm0=time[rates_total-1];
         mes=mes+" buy";
      }
      if(dn[rates_total-Alerts]!=EMPTY_VALUE && 
         dn[rates_total-Alerts-1]==EMPTY_VALUE && 
         tm1!=time[rates_total-1]
      ){
         tm1=time[rates_total-1];
         mes=mes+" sell";
      } 
      if(mes!=""){
         Alert("iUniTrend("+Symbol()+","+IntegerToString(PeriodSeconds()/60)+"):"+mes);
      }        
   }
   
   return(rv);

}
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer(){
   
}
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam){
                  
   
}
//+------------------------------------------------------------------+

void PrintColl(string caption,string message){
   Print(caption);
   string res[];
   int cnt=StringSplit(message,',',res);
   for(int i=0;i<cnt;i++){
      StringTrimLeft(res[i]);
      Print(res[i]);
   }
}

void PrepareParameters(){

   _Type=Type;
   
   if(UseDefault && KeepPrev){
      Parameters.f_dot2shift=PARAMETER_EMPTY;
      Parameters.f_level=PARAMETER_EMPTY;
      Parameters.f_method=PARAMETER_EMPTY;
      Parameters.f_period1=PARAMETER_EMPTY;
      Parameters.f_period2=PARAMETER_EMPTY;
      Parameters.f_period3=PARAMETER_EMPTY;
      Parameters.f_price=PARAMETER_EMPTY;
      Parameters.f_shift1=PARAMETER_EMPTY;
      Parameters.f_shift2=PARAMETER_EMPTY;
      Parameters.f_shift3=PARAMETER_EMPTY;
      Parameters.f_sto_price=PARAMETER_EMPTY;
      Parameters.f_volume=PARAMETER_EMPTY;
      Parameters.f_step=PARAMETER_EMPTY;
      Parameters.f_maximum=PARAMETER_EMPTY;
      Parameters.s_method=PARAMETER_EMPTY;
      Parameters.s_period1=PARAMETER_EMPTY;
      Parameters.s_period2=PARAMETER_EMPTY;
      Parameters.s_period3=PARAMETER_EMPTY;
      Parameters.s_price=PARAMETER_EMPTY;
      Parameters.s_shift1=PARAMETER_EMPTY;
      Parameters.s_shift2=PARAMETER_EMPTY;
      Parameters.s_shift3=PARAMETER_EMPTY;
   }
   else{   
      Parameters.f_dot2shift=f_dot2shift;
      Parameters.f_level=f_level;
      Parameters.f_method=f_method;
      Parameters.f_period1=f_period1;
      Parameters.f_period2=f_period2;
      Parameters.f_period3=f_period3;
      Parameters.f_price=f_price;
      Parameters.f_shift1=f_shift1;
      Parameters.f_shift2=f_shift2;
      Parameters.f_shift3=f_shift3;
      Parameters.f_sto_price=f_sto_price;
      Parameters.f_volume=f_volume;
      Parameters.f_step=f_step;
      Parameters.f_maximum=f_maximum;      
      Parameters.s_method=s_method;
      Parameters.s_period1=s_period1;
      Parameters.s_period2=s_period2;
      Parameters.s_period3=s_period3;
      Parameters.s_price=s_price;
      Parameters.s_shift1=s_shift1;
      Parameters.s_shift2=s_shift2;
      Parameters.s_shift3=s_shift3;
   }

}

void LoadIndicator(){
   switch(Type){
      case Type_ADX_PDIMDI:
         Indicator=new CUniTrend_ADX_PDIMDI(UseDefault,KeepPrev,Parameters);
      break;
      case Type_ADX_Level:
         Indicator=new CUniTrend_ADX_Level(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_ADXW_PDIMDI:
         Indicator=new CUniTrend_ADXW_PDIMDI(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_ADXW_Level:
         Indicator=new CUniTrend_ADXW_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_Alligator:
         Indicator=new CUniTrend_Alligator(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_AMA_Dir:
         Indicator=new CUniTrend_AMA_Dir(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_AMA_2MA:
         Indicator=new CUniTrend_AMA_2MA(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_ATR_Level:
         Indicator=new CUniTrend_ATR_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_BuBe_Level:
         Indicator=new CUniTrend_BuBe_Level(UseDefault,KeepPrev,Parameters);          
      break;
      case Type_BuBe_Dir:
         Indicator=new CUniTrend_BuBe_Dir(UseDefault,KeepPrev,Parameters);         
      break;
      case Type_Bands:
         Indicator=new CUniTrend_Bands(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_CCI_Level:
         Indicator=new CUniTrend_CCI_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_Chaikin_Level:
         Indicator=new CUniTrend_Chaikin_Level(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_DEMA_Dir:
         Indicator=new CUniTrend_DEMA_Dir(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_DEMA_2MA:
         Indicator=new CUniTrend_DEMA_2MA(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_DeMarker_Level:
         Indicator=new CUniTrend_DeMarker_Level(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_Envelopes:
         Indicator=new CUniTrend_Envelopes(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_Force_Level:
         Indicator=new CUniTrend_Force_Level(UseDefault,KeepPrev,Parameters);            
      break;
      case Type_Fractals:
         Indicator=new CUniTrend_Fractals();       
      break;
      case Type_FrAMA_Dir:
         Indicator=new CUniTrend_FrAMA_Dir(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_FrAMA_2MA:
         Indicator=new CUniTrend_FrAMA_2MA(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_Ichimoku_TK:
         Indicator=new CUniTrend_Ichimoku_TK(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_Ichimoku_SASB:
         Indicator=new CUniTrend_Ichimoku_SASB(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_Momentum_Level:
         Indicator=new CUniTrend_Momentum_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_MFI_Level:
         Indicator=new CUniTrend_MFI_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_MA_Dir:
         Indicator=new CUniTrend_MA_Dir(UseDefault,KeepPrev,Parameters); 
      break;
      case Type_MA_2MA:
         Indicator=new CUniTrend_MA_2MA(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_OsMA_Dir:
         Indicator=new CUniTrend_OsMA_Dir(UseDefault,KeepPrev,Parameters);  
      break;
      case Type_OsMA_Level:
         Indicator=new CUniTrend_OsMA_Level(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_MACD_Dir:
         Indicator=new CUniTrend_MACD_Dir(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_MACD_Level:
         Indicator=new CUniTrend_MACD_Level(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_SAR:
         Indicator=new CUniTrend_SAR(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_RSI_Level:
         Indicator=new CUniTrend_RSI_Level(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_RVI_Level:
         Indicator=new CUniTrend_RVI_Level(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_RVI_MS:
         Indicator=new CUniTrend_RVI_MS(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_STD_Level:
         Indicator=new CUniTrend_STD_Level(UseDefault,KeepPrev,Parameters); 
      break;
      case Type_Sto_Dir:
         Indicator=new CUniTrend_Stochastic_Dir(UseDefault,KeepPrev,Parameters);         
      break;      
      case Type_Sto_Level:
         Indicator=new CUniTrend_Stochastic_Level(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_Sto_MS:
         Indicator=new CUniTrend_Stochastic_MS(UseDefault,KeepPrev,Parameters);        
      break;
      case Type_TEMA_Dir:
         Indicator=new CUniTrend_TEMA_Dir(UseDefault,KeepPrev,Parameters);
      break;
      case Type_TEMA_2MA:
         Indicator=new CUniTrend_TEMA_2MA(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_TriX_Dir:
         Indicator=new CUniTrend_TriX_Dir(UseDefault,KeepPrev,Parameters);
      break;
      case Type_TriX_2MA:
         Indicator=new CUniTrend_TriX_2MA(UseDefault,KeepPrev,Parameters); 
      break;
      case Type_VIDyA_Dir:
         Indicator=new CUniTrend_VIDyA_Dir(UseDefault,KeepPrev,Parameters);      
      break;
      case Type_VIDyA_2MA:
         Indicator=new CUniTrend_VIDyA_2MA(UseDefault,KeepPrev,Parameters);       
      break;
      case Type_Volumes:
         Indicator=new CUniTrend_Volumes(UseDefault,KeepPrev,Parameters);       
      break;
   }
}